@@ -1,11 +1,19 @@
 #!/usr/bin/env bash
 
+saveenv=$-
+set -ex
+
+
 # devstack plugin for octavia
 
 GET_PIP_CACHE_LOCATION=/opt/stack/cache/files/get-pip.py
 
 function octavia_install {
-    setup_develop $OCTAVIA_DIR
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
+        setup_develop $OCTAVIA_DIR redis
+    else
+        setup_develop $OCTAVIA_DIR
+    fi
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
             if [[ ${DISTRO} =~ (rhel|centos) ]]; then
@@ -30,6 +38,8 @@ function octavia_lib_install {
     if use_library_from_git "octavia-lib"; then
         git_clone_by_name "octavia-lib"
         setup_dev_lib "octavia-lib"
+        export DIB_REPOLOCATION_octavia_lib=${GITDIR["octavia-lib"]}
+        export DIB_REPOREF_octavia_lib=$(git --git-dir="${GITDIR["octavia-lib"]}/.git" log -1 --pretty="format:%H")
     else
         pip_install_gr octavia-lib
     fi
@@ -42,6 +52,8 @@ function install_diskimage_builder {
         GITBRANCH["diskimage-builder"]=$DISKIMAGE_BUILDER_REPO_REF
         git_clone_by_name "diskimage-builder"
         setup_dev_lib -bindep "diskimage-builder"
+    else
+        pip_install -r $OCTAVIA_DIR/diskimage-create/requirements.txt
     fi
 }
 
@@ -60,6 +72,11 @@ function build_octavia_worker_image {
         export DIB_LOCAL_ELEMENTS=$DIB_LOCAL_ELEMENTS
     fi
 
+    # Pull in the option to install nftables in the amphora
+    if [ -n "$DIB_OCTAVIA_AMP_USE_NFTABLES" ]; then
+        export DIB_OCTAVIA_AMP_USE_NFTABLES=$DIB_OCTAVIA_AMP_USE_NFTABLES
+    fi
+
     # pull the agent code from the current code zuul has a reference to
     if [ -n "$DIB_REPOLOCATION_pip_and_virtualenv" ]; then
         export DIB_REPOLOCATION_pip_and_virtualenv=$DIB_REPOLOCATION_pip_and_virtualenv
@@ -108,21 +125,14 @@ function build_octavia_worker_image {
         fi
         sudo mkdir -m755 ${dib_logs}
         sudo chown $STACK_USER ${dib_logs}
-        # Build amphora image with master DIB in a Python 3 virtual environment
-        (
-            DIB_VENV_DIR=$(mktemp -d)
-            DIB_GIT_DIR=/tmp/dib-octavia
-            python3 -m venv $DIB_VENV_DIR
-            export USE_PYTHON3=True
-            export PATH=$DIB_VENV_DIR/bin:$PATH
-            if ! [ -d $DIB_GIT_DIR ]; then
-                git clone ${GITREPO["diskimage-builder"]} $DIB_GIT_DIR
-            fi
-            (cd $REQUIREMENTS_DIR && git show origin/master:upper-constraints.txt) | sed '/diskimage-builder/d' > $DIB_VENV_DIR/u-c.txt
-            pip install -c $DIB_VENV_DIR/u-c.txt $DIB_GIT_DIR
-            $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
-        )
+        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
     fi
+
+    if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
+        echo "Diskimage-builder failed to create the amphora image. Aborting."
+        exit 1
+    fi
+
     upload_image file://${OCTAVIA_AMP_IMAGE_FILE} $TOKEN
 
 }
@@ -189,7 +199,7 @@ function _cleanup_octavia_apache_wsgi {
 
 function _start_octavia_apache_wsgi {
     if [[ "$WSGI_MODE" == "uwsgi" ]]; then
-        run_process o-api "$OCTAVIA_BIN_DIR/uwsgi --ini $OCTAVIA_UWSGI_CONF"
+        run_process o-api "$(which uwsgi) --ini $OCTAVIA_UWSGI_CONF"
         enable_apache_site octavia-wsgi
     else
         enable_apache_site octavia
@@ -246,11 +256,6 @@ function install_redis {
 
     start_service redis
 
-    pip_install_gr redis
-}
-
-function stop_redis {
-    stop_service redis || true
 }
 
 function uninstall_redis {
@@ -262,7 +267,8 @@ function uninstall_redis {
         uninstall_package redis
     fi
 
-    pip_unistall redis
+    stop_service redis
+
 }
 
 function octavia_configure {
@@ -290,7 +296,7 @@ function octavia_configure {
     iniset $OCTAVIA_CONF api_settings api_handler queue_producer
 
     iniset $OCTAVIA_CONF database connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia"
-    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_PROVIDER} == True ]]; then
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
         iniset $OCTAVIA_CONF task_flow persistence_connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia_persistence"
     fi
     # Configure keystone auth_token for all users
@@ -312,6 +318,7 @@ function octavia_configure {
     iniset $OCTAVIA_CONF controller_worker compute_driver ${OCTAVIA_COMPUTE_DRIVER}
     iniset $OCTAVIA_CONF controller_worker volume_driver  ${OCTAVIA_VOLUME_DRIVER}
     iniset $OCTAVIA_CONF controller_worker network_driver ${OCTAVIA_NETWORK_DRIVER}
+    iniset $OCTAVIA_CONF controller_worker image_driver ${OCTAVIA_IMAGE_DRIVER}
     iniset $OCTAVIA_CONF controller_worker amp_image_tag ${OCTAVIA_AMP_IMAGE_TAG}
 
     iniuncomment $OCTAVIA_CONF health_manager heartbeat_key
@@ -367,7 +374,7 @@ function octavia_configure {
         recreate_database_mysql octavia
         octavia-db-manage upgrade head
 
-        if [[ ${OCTAVIA_ENABLE_AMPHORAV2_PROVIDER} == True ]]; then
+        if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
             recreate_database_mysql octavia_persistence
             octavia-db-manage upgrade_persistence
         fi
@@ -378,7 +385,7 @@ function octavia_configure {
     fi
 
     # amphorav2 required redis installation
-    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_PROVIDER} == True ]]; then
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
         install_redis
     fi
 
@@ -408,7 +415,7 @@ function octavia_configure {
     iniset $OCTAVIA_CONF certificates server_certs_key_passphrase insecure-key-do-not-use-this-key
 
     if [[ "$OCTAVIA_USE_LEGACY_RBAC" == "True" ]]; then
-        cp $OCTAVIA_DIR/etc/policy/admin_or_owner-policy.json $OCTAVIA_CONF_DIR/policy.json
+        cp $OCTAVIA_DIR/etc/policy/admin_or_owner-policy.yaml $OCTAVIA_CONF_DIR/policy.yaml
     fi
 
     # create dhclient.conf file for dhclient
@@ -468,18 +475,35 @@ function create_mgmt_network_interface {
         die "Unknown network controller. Please define octavia_create_network_interface_device"
     fi
     sudo ip link set dev o-hm0 address $MGMT_PORT_MAC
-    if [ $SERVICE_IP_VERSION == '6' ] ; then
-        # Allow the required IPv6 ICMP messages
-        sudo ip6tables -I INPUT -i o-hm0 -p ipv6-icmp -j ACCEPT
-        sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_HM_LISTEN_PORT -j ACCEPT
-        sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_ADMIN_PORT -j ACCEPT
-        sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_TENANT_PORT -j ACCEPT
+
+    # Check if the host is using nftables, an alternative to iptables
+    if [ -x "$(sudo bash -c 'command -v nft')" ]; then
+        sudo nft add table inet octavia
+        sudo nft add chain inet octavia o-hm0-incoming { type filter hook input priority 0\;}
+        sudo nft flush chain inet octavia o-hm0-incoming
+        # Note: Order is important here and using counter here as this is
+        # devstack for testing.
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" counter log drop
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" meta l4proto ipv6-icmp counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" udp dport $OCTAVIA_HM_LISTEN_PORT counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" udp dport $OCTAVIA_AMP_LOG_ADMIN_PORT counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" udp dport $OCTAVIA_AMP_LOG_TENANT_PORT counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" ct state related,established accept
     else
-        sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_HM_LISTEN_PORT -j ACCEPT
-        sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_ADMIN_PORT -j ACCEPT
-        sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_TENANT_PORT -j ACCEPT
+        if [ $SERVICE_IP_VERSION == '6' ] ; then
+            # Allow the required IPv6 ICMP messages
+            sudo ip6tables -I INPUT -i o-hm0 -p ipv6-icmp -j ACCEPT
+            sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_HM_LISTEN_PORT -j ACCEPT
+            sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_ADMIN_PORT -j ACCEPT
+            sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_TENANT_PORT -j ACCEPT
+        else
+            sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_HM_LISTEN_PORT -j ACCEPT
+            sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_ADMIN_PORT -j ACCEPT
+            sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_TENANT_PORT -j ACCEPT
+        fi
     fi
 
+
     if [ $OCTAVIA_CONTROLLER_IP_PORT_LIST == 'auto' ] ; then
         iniset $OCTAVIA_CONF health_manager controller_ip_port_list $MGMT_PORT_IP:$OCTAVIA_HM_LISTEN_PORT
     else
@@ -503,6 +527,9 @@ function build_mgmt_network {
     openstack network create lb-mgmt-net
     if [ $SERVICE_IP_VERSION == '6' ] ; then
         openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET_IPV6 --allocation-pool start=$OCTAVIA_MGMT_SUBNET_IPV6_START,end=$OCTAVIA_MGMT_SUBNET_IPV6_END --network lb-mgmt-net --ip-version 6 --ipv6-address-mode slaac --ipv6-ra-mode slaac lb-mgmt-subnet
+        # SLAAC needs a router on the subnet to advertise the prefix.
+        openstack router create lb-mgmt-router
+        openstack router add subnet lb-mgmt-router lb-mgmt-subnet
     else
         openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet
     fi
@@ -631,10 +658,6 @@ function octavia_stop {
     else
         die "Unknown network controller. Please define octavia_delete_network_interface_device"
     fi
-
-    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_PROVIDER} == True ]]; then
-        stop_redis
-    fi
 }
 
 function octavia_cleanup {
@@ -659,7 +682,7 @@ function octavia_cleanup {
     fi
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         if [ ${OCTAVIA_AMP_SSH_KEY_NAME}x != x ] ; then
-            openstack keypair delete ${OCTAVIA_AMP_SSH_KEY_NAME}
+            openstack keypair delete ${OCTAVIA_AMP_SSH_KEY_NAME} || true
         fi
     fi
     if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
@@ -671,7 +694,7 @@ function octavia_cleanup {
 
     sudo rm -rf $NOVA_STATE_PATH $NOVA_AUTH_CACHE_DIR
 
-    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_PROVIDER} == True ]]; then
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
         uninstall_redis
     fi
 
@@ -812,3 +835,14 @@ if [[ "$1" == "clean" ]]; then
         octavia_cleanup
     fi
 fi
+
+if [[ $saveenv =~ e ]]; then
+    set -e
+else
+    set +e
+fi
+if [[ $saveenv =~ x ]]; then
+    set -x
+else
+    set +x
+fi
