@@ -45,10 +45,20 @@ AMPHORA_SUPPORTED_LB_ALGORITHMS = [
     consts.LB_ALGORITHM_SOURCE_IP,
     consts.LB_ALGORITHM_LEAST_CONNECTIONS]
 
+AMPHORA_SUPPORTED_PROTOCOLS = [
+    lib_consts.PROTOCOL_TCP,
+    lib_consts.PROTOCOL_HTTP,
+    lib_consts.PROTOCOL_HTTPS,
+    lib_consts.PROTOCOL_TERMINATED_HTTPS,
+    lib_consts.PROTOCOL_PROXY,
+    lib_consts.PROTOCOL_PROXYV2,
+    lib_consts.PROTOCOL_UDP,
+]
+
 
 class AmphoraProviderDriver(driver_base.ProviderDriver):
     def __init__(self):
-        super(AmphoraProviderDriver, self).__init__()
+        super().__init__()
         self.target = messaging.Target(
             namespace=consts.RPC_NAMESPACE_CONTROLLER_AGENT,
             topic=consts.TOPIC_AMPHORA_V2, version="2.0", fanout=False)
@@ -65,6 +75,29 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                 user_fault_string=msg,
                 operator_fault_string=msg)
 
+    def _validate_listener_protocol(self, listener):
+        if listener.protocol not in AMPHORA_SUPPORTED_PROTOCOLS:
+            msg = ('Amphora provider does not support %s protocol. '
+                   'Supported: %s'
+                   % (listener.protocol,
+                      ", ".join(AMPHORA_SUPPORTED_PROTOCOLS)))
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
+
+    def _validate_alpn_protocols(self, listener):
+        if not listener.alpn_protocols:
+            return
+        supported = consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS
+        not_supported = set(listener.alpn_protocols) - set(supported)
+        if not_supported:
+            msg = ('Amphora provider does not support %s ALPN protocol(s). '
+                   'Supported: %s'
+                   % (", ".join(not_supported), ", ".join(supported)))
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
+
     # Load Balancer
     def create_vip_port(self, loadbalancer_id, project_id, vip_dictionary):
         vip_obj = driver_utils.provider_vip_dict_to_vip_obj(vip_dictionary)
@@ -160,6 +193,8 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
     # Listener
     def listener_create(self, listener):
+        self._validate_listener_protocol(listener)
+        self._validate_alpn_protocols(listener)
         payload = {consts.LISTENER: listener.to_dict()}
         self._encrypt_listener_dict(payload[consts.LISTENER])
 
@@ -170,6 +205,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         self.client.cast({}, 'delete_listener', **payload)
 
     def listener_update(self, old_listener, new_listener):
+        self._validate_alpn_protocols(new_listener)
         original_listener = old_listener.to_dict()
         listener_updates = new_listener.to_dict()
 
@@ -273,10 +309,13 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             if m.id not in new_member_ids:
                 deleted_members.append(m)
 
-        payload = {'old_members': [m.to_dict() for m in deleted_members],
-                   'new_members': [m.to_dict() for m in new_members],
-                   'updated_members': updated_members}
-        self.client.cast({}, 'batch_update_members', **payload)
+        if deleted_members or new_members or updated_members:
+            payload = {'old_members': [m.to_dict() for m in deleted_members],
+                       'new_members': [m.to_dict() for m in new_members],
+                       'updated_members': updated_members}
+            self.client.cast({}, 'batch_update_members', **payload)
+        else:
+            LOG.info("Member batch update is a noop, returning early.")
 
     def _validate_members(self, db_pool, members):
         if db_pool.protocol == consts.PROTOCOL_UDP:
@@ -419,6 +458,26 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             # when the octavia-lib supports it.
             compute_driver.validate_flavor(compute_flavor)
 
+        amp_image_tag = flavor_dict.get(consts.AMP_IMAGE_TAG, None)
+        if amp_image_tag:
+            image_driver = stevedore_driver.DriverManager(
+                namespace='octavia.image.drivers',
+                name=CONF.controller_worker.image_driver,
+                invoke_on_load=True
+            ).driver
+
+            try:
+                image_driver.get_image_id_by_tag(
+                    amp_image_tag, CONF.controller_worker.amp_image_owner_id)
+            except Exception as e:
+                raise exceptions.NotFound(
+                    user_fault_string='Failed to find an image with tag {} '
+                                      'due to: {}'.format(
+                                          amp_image_tag, str(e)),
+                    operator_fault_string='Failed to find an image with tag '
+                                          '{} due to: {}'.format(
+                                              amp_image_tag, str(e)))
+
     # Availability Zone
     def get_supported_availability_zone_metadata(self):
         """Returns the valid availability zone metadata keys and descriptions.
