@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from octavia_lib.api.drivers import data_models as driver_dm
+from octavia_lib.common import constants as lib_consts
 from oslo_config import cfg
 from oslo_db import exception as odb_exceptions
 from oslo_log import log as logging
@@ -22,7 +24,6 @@ from pecan import request as pecan_request
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
-from octavia.api.drivers import data_models as driver_dm
 from octavia.api.drivers import driver_factory
 from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
@@ -46,7 +47,7 @@ class PoolsController(base.BaseController):
     RBAC_TYPE = constants.RBAC_POOL
 
     def __init__(self):
-        super(PoolsController, self).__init__()
+        super().__init__()
 
     @wsme_pecan.wsexpose(pool_types.PoolRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
@@ -122,17 +123,32 @@ class PoolsController(base.BaseController):
                 pool_dict.get('ca_tls_certificate_id'),
                 pool_dict.get('crl_container_id', None))
 
+        # Check TLS cipher prohibit list
+        if 'tls_ciphers' in pool_dict and pool_dict['tls_ciphers']:
+            rejected_ciphers = validate.check_cipher_prohibit_list(
+                pool_dict['tls_ciphers'])
+            if rejected_ciphers:
+                raise exceptions.ValidationException(detail=_(
+                    'The following ciphers have been prohibited by an '
+                    'administrator: ' + ', '.join(rejected_ciphers)))
+
+        if pool_dict['tls_enabled']:
+            # Validate TLS version list
+            validate.check_tls_version_list(pool_dict['tls_versions'])
+            # Validate TLS versions against minimum
+            validate.check_tls_version_min(pool_dict['tls_versions'])
+
         try:
             return self.repositories.create_pool_on_load_balancer(
                 lock_session, pool_dict,
                 listener_id=listener_id)
-        except odb_exceptions.DBDuplicateEntry:
-            raise exceptions.IDAlreadyExists()
-        except odb_exceptions.DBError:
+        except odb_exceptions.DBDuplicateEntry as e:
+            raise exceptions.IDAlreadyExists() from e
+        except odb_exceptions.DBError as e:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
             # do not give any information as to what constraint failed
-            raise exceptions.InvalidOption(value='', option='')
+            raise exceptions.InvalidOption(value='', option='') from e
 
     def _is_only_specified_in_request(self, request, **kwargs):
         request_attrs = []
@@ -150,7 +166,7 @@ class PoolsController(base.BaseController):
                 return False
         return True
 
-    def _validate_pool_request_for_udp(self, request):
+    def _validate_pool_request_for_udp_sctp(self, request):
         if request.session_persistence:
             if (request.session_persistence.type ==
                     constants.SESSION_PERSISTENCE_SOURCE_IP and
@@ -159,14 +175,15 @@ class PoolsController(base.BaseController):
                         check_exist_attrs=['type', 'persistence_timeout',
                                            'persistence_granularity'])):
                 raise exceptions.ValidationException(detail=_(
-                    "session_persistence %s type for UDP protocol "
+                    "session_persistence %s type for UDP and SCTP protocols "
                     "only accepts: type, persistence_timeout, "
                     "persistence_granularity.") % (
                         constants.SESSION_PERSISTENCE_SOURCE_IP))
             if request.session_persistence.cookie_name:
                 raise exceptions.ValidationException(detail=_(
                     "Cookie names are not supported for %s pools.") %
-                    constants.PROTOCOL_UDP)
+                    "/".join((constants.PROTOCOL_UDP,
+                              lib_consts.PROTOCOL_SCTP)))
             if request.session_persistence.type in [
                 constants.SESSION_PERSISTENCE_HTTP_COOKIE,
                     constants.SESSION_PERSISTENCE_APP_COOKIE]:
@@ -174,7 +191,8 @@ class PoolsController(base.BaseController):
                     "Session persistence of type %(type)s is not supported "
                     "for %(protocol)s protocol pools.") % {
                     'type': request.session_persistence.type,
-                    'protocol': constants.PROTOCOL_UDP})
+                    'protocol': "/".join((constants.PROTOCOL_UDP,
+                                          lib_consts.PROTOCOL_SCTP))})
 
     @wsme_pecan.wsexpose(pool_types.PoolRootResponse,
                          body=pool_types.PoolRootPOST, status_code=201)
@@ -211,15 +229,16 @@ class PoolsController(base.BaseController):
         if pool.listener_id and listener:
             self._validate_protocol(listener.protocol, pool.protocol)
 
-        if pool.protocol == constants.PROTOCOL_UDP:
-            self._validate_pool_request_for_udp(pool)
+        if pool.protocol in (constants.PROTOCOL_UDP,
+                             lib_consts.PROTOCOL_SCTP):
+            self._validate_pool_request_for_udp_sctp(pool)
         else:
             if (pool.session_persistence and (
                     pool.session_persistence.persistence_timeout or
                     pool.session_persistence.persistence_granularity)):
                 raise exceptions.ValidationException(detail=_(
                     "persistence_timeout and persistence_granularity "
-                    "is only for UDP protocol pools."))
+                    "is only for UDP and SCTP protocol pools."))
 
         if pool.session_persistence:
             sp_dict = pool.session_persistence.to_dict(render_unsets=False)
@@ -296,16 +315,20 @@ class PoolsController(base.BaseController):
             hm[constants.PROJECT_ID] = db_pool.project_id
             new_hm = health_monitor.HealthMonitorController()._graph_create(
                 lock_session, hm)
-            if db_pool.protocol == constants.PROTOCOL_UDP:
+            if db_pool.protocol in (constants.PROTOCOL_UDP,
+                                    lib_consts.PROTOCOL_SCTP):
                 health_monitor.HealthMonitorController(
-                )._validate_healthmonitor_request_for_udp(new_hm)
+                )._validate_healthmonitor_request_for_udp_sctp(new_hm,
+                                                               db_pool)
             else:
-                if new_hm.type == constants.HEALTH_MONITOR_UDP_CONNECT:
+                if new_hm.type in (constants.HEALTH_MONITOR_UDP_CONNECT,
+                                   lib_consts.HEALTH_MONITOR_SCTP):
                     raise exceptions.ValidationException(detail=_(
                         "The %(type)s type is only supported for pools of "
                         "type %(protocol)s.") % {
                             'type': new_hm.type,
-                            'protocol': constants.PROTOCOL_UDP})
+                            'protocol': '/'.join((constants.PROTOCOL_UDP,
+                                                  lib_consts.PROTOCOL_SCTP))})
             db_pool.health_monitor = new_hm
 
         # Now check quotas for members
@@ -329,8 +352,9 @@ class PoolsController(base.BaseController):
 
     def _validate_pool_PUT(self, pool, db_pool):
 
-        if db_pool.protocol == constants.PROTOCOL_UDP:
-            self._validate_pool_request_for_udp(pool)
+        if db_pool.protocol in (constants.PROTOCOL_UDP,
+                                lib_consts.PROTOCOL_SCTP):
+            self._validate_pool_request_for_udp_sctp(pool)
         else:
             if (pool.session_persistence and (
                     pool.session_persistence.persistence_timeout or
@@ -381,6 +405,21 @@ class PoolsController(base.BaseController):
         if ca_ref:
             self._validate_client_ca_and_crl_refs(ca_ref, crl_ref)
 
+        # Check TLS cipher prohibit list
+        if pool.tls_ciphers:
+            rejected_ciphers = validate.check_cipher_prohibit_list(
+                pool.tls_ciphers)
+            if rejected_ciphers:
+                raise exceptions.ValidationException(detail=_(
+                    "The following ciphers have been prohibited by an "
+                    "administrator: " + ', '.join(rejected_ciphers)))
+
+        if pool.tls_versions is not wtypes.Unset:
+            # Validate TLS version list
+            validate.check_tls_version_list(pool.tls_versions)
+            # Validate TLS version against minimum
+            validate.check_tls_version_min(pool.tls_versions)
+
     @wsme_pecan.wsexpose(pool_types.PoolRootResponse, wtypes.text,
                          body=pool_types.PoolRootPut, status_code=200)
     def put(self, id, pool_):
@@ -394,6 +433,11 @@ class PoolsController(base.BaseController):
 
         self._auth_validate_action(context, project_id, constants.RBAC_PUT)
 
+        if pool.tls_versions is None:
+            pool.tls_versions = CONF.api_settings.default_pool_tls_versions
+        if pool.tls_ciphers is None:
+            pool.tls_ciphers = CONF.api_settings.default_pool_ciphers
+
         if (pool.session_persistence and
                 not pool.session_persistence.type and
                 db_pool.session_persistence and
