@@ -37,7 +37,7 @@ class BaseNetworkTask(task.Task):
     """Base task to load drivers common to the tasks."""
 
     def __init__(self, **kwargs):
-        super(BaseNetworkTask, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self._network_driver = None
         self.task_utils = task_utils.TaskUtils()
         self.lb_repo = repositories.LoadBalancerRepository()
@@ -59,10 +59,10 @@ class CalculateAmphoraDelta(BaseNetworkTask):
 
         if vrrp_port is None:
             vrrp_port = self.network_driver.get_port(amphora.vrrp_port_id)
-        if (availability_zone and
-                availability_zone.get(constants.MANAGEMENT_NETWORK)):
-            management_nets = [availability_zone.get(
-                constants.MANAGEMENT_NETWORK)]
+        if availability_zone:
+            management_nets = (
+                [availability_zone.get(constants.MANAGEMENT_NETWORK)] or
+                CONF.controller_worker.amp_boot_network_list)
         else:
             management_nets = CONF.controller_worker.amp_boot_network_list
         desired_network_ids = {vrrp_port.network_id}.union(management_nets)
