@@ -38,7 +38,7 @@ class BaseNetworkTask(task.Task):
     """Base task to load drivers common to the tasks."""
 
     def __init__(self, **kwargs):
-        super(BaseNetworkTask, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self._network_driver = None
         self.task_utils = task_utils.TaskUtils()
         self.loadbalancer_repo = repo.LoadBalancerRepository()
@@ -69,10 +69,10 @@ class CalculateAmphoraDelta(BaseNetworkTask):
 
         # Figure out what networks we want
         # seed with lb network(s)
-        if (availability_zone and
-                availability_zone.get(constants.MANAGEMENT_NETWORK)):
-            management_nets = [
-                availability_zone.get(constants.MANAGEMENT_NETWORK)]
+        if availability_zone:
+            management_nets = (
+                [availability_zone.get(constants.MANAGEMENT_NETWORK)] or
+                CONF.controller_worker.amp_boot_network_list)
         else:
             management_nets = CONF.controller_worker.amp_boot_network_list
         desired_network_ids = {vrrp_port_network_id}.union(management_nets)
@@ -405,7 +405,7 @@ class UpdateVIPSecurityGroup(BaseNetworkTask):
         LOG.debug("Setup SG for loadbalancer id: %s", loadbalancer_id)
         db_lb = self.loadbalancer_repo.get(
             db_apis.get_session(), id=loadbalancer_id)
-        return self.network_driver.update_vip_sg(db_lb, db_lb.vip)
+        self.network_driver.update_vip_sg(db_lb, db_lb.vip)
 
 
 class GetSubnetFromVIP(BaseNetworkTask):
@@ -452,13 +452,12 @@ class PlugVIPAmphora(BaseNetworkTask):
                                            id=amphora.get(constants.ID))
             db_amp.vrrp_port_id = result[constants.VRRP_PORT_ID]
             db_amp.ha_port_id = result[constants.HA_PORT_ID]
-            db_subnet = self.network_driver.get_subnet(subnet[constants.ID])
             db_lb = self.loadbalancer_repo.get(
                 db_apis.get_session(),
                 id=loadbalancer[constants.LOADBALANCER_ID])
 
             self.network_driver.unplug_aap_port(db_lb.vip,
-                                                db_amp, db_subnet)
+                                                db_amp, subnet)
         except Exception as e:
             LOG.error('Failed to unplug AAP port. Resources may still be in '
                       'use for VIP: %s due to error: %s', db_lb.vip, str(e))
