@@ -29,7 +29,7 @@ from octavia.db import models
 class OctaviaDBTestBase(test_base.DbTestCase):
 
     def setUp(self, connection_string='sqlite://'):
-        super(OctaviaDBTestBase, self).setUp()
+        super().setUp()
         # NOTE(blogan): doing this for now because using the engine and
         # session set up in the fixture for test_base.DbTestCase does not work
         # with the API functional tests.  Need to investigate more if this
