@@ -26,7 +26,7 @@ from octavia.tests.unit import base
 
 class TestAmphoraDriver(base.TestRpc):
     def setUp(self):
-        super(TestAmphoraDriver, self).setUp()
+        super().setUp()
         self.amp_driver = driver.AmphoraProviderDriver()
         self.sample_data = sample_data_models.SampleDriverDataModels()
 
@@ -137,11 +137,36 @@ class TestAmphoraDriver(base.TestRpc):
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_listener_create(self, mock_cast):
         provider_listener = driver_dm.Listener(
-            listener_id=self.sample_data.listener1_id)
+            listener_id=self.sample_data.listener1_id,
+            protocol=consts.PROTOCOL_HTTPS,
+            alpn_protocols=consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS)
         self.amp_driver.listener_create(provider_listener)
         payload = {consts.LISTENER_ID: self.sample_data.listener1_id}
         mock_cast.assert_called_with({}, 'create_listener', **payload)
 
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_listener_create_unsupported_alpn(self, mock_cast):
+        provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id,
+            protocol=consts.PROTOCOL_HTTPS)
+        provider_listener.alpn_protocols = ['http/1.1', 'eureka']
+        self.assertRaises(
+            exceptions.UnsupportedOptionError,
+            self.amp_driver.listener_create,
+            provider_listener)
+        mock_cast.assert_not_called()
+
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_listener_create_unsupported_protocol(self, mock_cast):
+        provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id,
+            protocol='UNSUPPORTED_PROTO')
+        self.assertRaises(
+            exceptions.UnsupportedOptionError,
+            self.amp_driver.listener_create,
+            provider_listener)
+        mock_cast.assert_not_called()
+
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_listener_delete(self, mock_cast):
         provider_listener = driver_dm.Listener(
@@ -176,6 +201,19 @@ class TestAmphoraDriver(base.TestRpc):
                    consts.LISTENER_UPDATES: listener_dict}
         mock_cast.assert_called_with({}, 'update_listener', **payload)
 
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_listener_update_unsupported_alpn(self, mock_cast):
+        old_provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id)
+        provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id,
+            alpn_protocols=['http/1.1', 'eureka'])
+        self.assertRaises(
+            exceptions.UnsupportedOptionError,
+            self.amp_driver.listener_update,
+            old_provider_listener,
+            provider_listener)
+
     # Pool
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_pool_create(self, mock_cast):
@@ -420,22 +458,13 @@ class TestAmphoraDriver(base.TestRpc):
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_member_batch_update_clear_already_empty(
             self, mock_cast, mock_pool_get, mock_session):
-        """Expect that we will pass an empty payload if directed.
-
-        Logic for whether or not to attempt this will be done above the driver
-        layer, so our driver is responsible to forward the request even if it
-        is a perceived no-op.
-        """
         mock_pool = mock.MagicMock()
         mock_pool_get.return_value = mock_pool
 
         self.amp_driver.member_batch_update(
             self.sample_data.pool1_id, [])
 
-        payload = {'old_member_ids': [],
-                   'new_member_ids': [],
-                   'updated_members': []}
-        mock_cast.assert_called_with({}, 'batch_update_members', **payload)
+        mock_cast.assert_not_called()
 
     # Health Monitor
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -709,8 +738,10 @@ class TestAmphoraDriver(base.TestRpc):
                 self.amp_driver.get_supported_availability_zone_metadata)
 
     def test_validate_availability_zone(self):
-        ref_dict = {consts.COMPUTE_ZONE: 'my_compute_zone'}
-        self.amp_driver.validate_availability_zone(ref_dict)
+        with mock.patch('stevedore.driver.DriverManager.driver') as m_driver:
+            m_driver.validate_availability_zone.return_value = None
+            ref_dict = {consts.COMPUTE_ZONE: 'my_compute_zone'}
+            self.amp_driver.validate_availability_zone(ref_dict)
 
         # Test bad availability zone metadata key
         ref_dict = {'bogus': 'bogus'}
