@@ -42,6 +42,21 @@ class TestAmphoraDriver(base.TestRpc):
 
         self.assertEqual(self.sample_data.provider_vip_dict, provider_vip_dict)
 
+    @mock.patch('octavia.common.utils.get_network_driver')
+    def test_create_vip_port_without_port_security_enabled(
+            self, mock_get_net_driver):
+        mock_net_driver = mock.MagicMock()
+        mock_get_net_driver.return_value = mock_net_driver
+        network = mock.MagicMock()
+        network.port_security_enabled = False
+        mock_net_driver.get_network.return_value = network
+        mock_net_driver.allocate_vip.return_value = self.sample_data.db_vip
+
+        self.assertRaises(exceptions.DriverError,
+                          self.amp_driver.create_vip_port,
+                          self.sample_data.lb_id, self.sample_data.project_id,
+                          self.sample_data.provider_vip_dict)
+
     @mock.patch('octavia.common.utils.get_network_driver')
     def test_create_vip_port_failed(self, mock_get_net_driver):
         mock_net_driver = mock.MagicMock()
@@ -123,6 +138,7 @@ class TestAmphoraDriver(base.TestRpc):
     def test_listener_create(self, mock_cast):
         provider_listener = driver_dm.Listener(
             listener_id=self.sample_data.listener1_id,
+            protocol=consts.PROTOCOL_HTTPS,
             alpn_protocols=consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS)
         self.amp_driver.listener_create(provider_listener)
         payload = {consts.LISTENER: provider_listener.to_dict()}
@@ -131,7 +147,8 @@ class TestAmphoraDriver(base.TestRpc):
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_listener_create_unsupported_alpn(self, mock_cast):
         provider_listener = driver_dm.Listener(
-            listener_id=self.sample_data.listener1_id)
+            listener_id=self.sample_data.listener1_id,
+            protocol=consts.PROTOCOL_HTTPS)
         provider_listener.alpn_protocols = ['http/1.1', 'eureka']
         self.assertRaises(
             exceptions.UnsupportedOptionError,
@@ -139,6 +156,17 @@ class TestAmphoraDriver(base.TestRpc):
             provider_listener)
         mock_cast.assert_not_called()
 
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_listener_create_unsupported_protocol(self, mock_cast):
+        provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id,
+            protocol='UNSUPPORTED_PROTO')
+        self.assertRaises(
+            exceptions.UnsupportedOptionError,
+            self.amp_driver.listener_create,
+            provider_listener)
+        mock_cast.assert_not_called()
+
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_listener_delete(self, mock_cast):
         provider_listener = driver_dm.Listener(
@@ -222,8 +250,19 @@ class TestAmphoraDriver(base.TestRpc):
         old_provider_pool = driver_dm.Pool(
             pool_id=self.sample_data.pool1_id)
         provider_pool = driver_dm.Pool(
-            pool_id=self.sample_data.pool1_id, admin_state_up=True)
-        pool_dict = {'enabled': True}
+            pool_id=self.sample_data.pool1_id, admin_state_up=True,
+            ca_tls_container_data='CA DATA', ca_tls_container_ref='CA REF',
+            crl_container_data='CRL DATA', crl_container_ref='CRL REF',
+            description='TEST DESCRIPTION', name='TEST NAME',
+            lb_algorithm=consts.LB_ALGORITHM_SOURCE_IP,
+            session_persistence='FAKE SP', tls_container_data='TLS DATA',
+            tls_container_ref='TLS REF', tls_enabled=False)
+        pool_dict = {'description': 'TEST DESCRIPTION',
+                     'lb_algorithm': 'SOURCE_IP', 'name': 'TEST NAME',
+                     'session_persistence': 'FAKE SP', 'tls_enabled': False,
+                     'enabled': True, 'tls_certificate_id': 'TLS REF',
+                     'ca_tls_certificate_id': 'CA REF',
+                     'crl_container_id': 'CRL REF'}
         self.amp_driver.pool_update(old_provider_pool, provider_pool)
         payload = {consts.ORIGINAL_POOL: old_provider_pool.to_dict(),
                    consts.POOL_UPDATES: pool_dict}
@@ -744,9 +783,9 @@ class TestAmphoraDriver(base.TestRpc):
     def test_encrypt_listener_dict(self, mock_fernet):
         mock_fern = mock.MagicMock()
         mock_fernet.return_value = mock_fern
-        TEST_DATA = 'some data'
-        TEST_DATA2 = 'more data'
-        FAKE_ENCRYPTED_DATA = 'alqwkhjetrhth'
+        TEST_DATA = {'cert': b'some data'}
+        TEST_DATA2 = {'test': 'more data'}
+        FAKE_ENCRYPTED_DATA = b'alqwkhjetrhth'
         mock_fern.encrypt.return_value = FAKE_ENCRYPTED_DATA
 
         # We need a class instance with the mock
@@ -757,21 +796,21 @@ class TestAmphoraDriver(base.TestRpc):
 
         amp_driver._encrypt_listener_dict(list_dict)
 
-        mock_fern.encrypt.assert_called_once_with(TEST_DATA)
+        mock_fern.encrypt.assert_called_once_with(b'some data')
 
-        self.assertEqual(FAKE_ENCRYPTED_DATA,
+        self.assertEqual({'cert': FAKE_ENCRYPTED_DATA},
                          list_dict[consts.DEFAULT_TLS_CONTAINER_DATA])
 
         mock_fern.reset_mock()
 
         # Test just sni_container_data
-        list_dict = {consts.SNI_CONTAINER_DATA: [TEST_DATA, TEST_DATA2]}
+        TEST_DATA = {'cert': b'some data'}
+        sni_dict = {consts.SNI_CONTAINER_DATA: [TEST_DATA, TEST_DATA2]}
 
-        amp_driver._encrypt_listener_dict(list_dict)
+        amp_driver._encrypt_listener_dict(sni_dict)
 
-        calls = [mock.call(TEST_DATA), mock.call(TEST_DATA2)]
+        mock_fern.encrypt.assert_called_once_with(b'some data')
 
-        mock_fern.encrypt.assert_has_calls(calls)
-
-        encrypted_sni = [FAKE_ENCRYPTED_DATA, FAKE_ENCRYPTED_DATA]
-        self.assertEqual(encrypted_sni, list_dict[consts.SNI_CONTAINER_DATA])
+        encrypted_sni = [{'cert': FAKE_ENCRYPTED_DATA},
+                         TEST_DATA2]
+        self.assertEqual(encrypted_sni, sni_dict[consts.SNI_CONTAINER_DATA])
