@@ -36,7 +36,6 @@ COMPUTE_ID = uuidutils.generate_uuid()
 PORT_ID = uuidutils.generate_uuid()
 SUBNET_ID = uuidutils.generate_uuid()
 NETWORK_ID = uuidutils.generate_uuid()
-SG_ID = uuidutils.generate_uuid()
 IP_ADDRESS = "172.24.41.1"
 VIP = o_data_models.Vip(port_id=t_constants.MOCK_PORT_ID,
                         subnet_id=t_constants.MOCK_SUBNET_ID,
@@ -105,7 +104,7 @@ class TestNetworkTasks(base.TestCase):
         conf.config(group="controller_worker",
                     amp_boot_network_list=[self.boot_net_id])
 
-        super(TestNetworkTasks, self).setUp()
+        super().setUp()
 
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     @mock.patch('octavia.db.api.get_session', return_value=_session_mock)
@@ -230,15 +229,11 @@ class TestNetworkTasks(base.TestCase):
                          calc_delta.execute(self.load_balancer_mock, {}))
 
         # Test with one amp and one pool and one member, nothing plugged
-        # Dummy AZ is provided
         # Delta should be one additional subnet to plug
         mock_driver.reset_mock()
         member_mock = mock.MagicMock()
         member_mock.subnet_id = 1
         pool_mock.members = [member_mock]
-        az = {
-            constants.COMPUTE_ZONE: 'foo'
-        }
         mock_driver.get_subnet.return_value = data_models.Subnet(id=2,
                                                                  network_id=3)
 
@@ -248,7 +243,7 @@ class TestNetworkTasks(base.TestCase):
                                     data_models.Interface(network_id=3)],
                                 delete_nics=[]).to_dict(recurse=True)
         self.assertEqual({self.db_amphora_mock.id: ndm},
-                         calc_delta.execute(self.load_balancer_mock, az))
+                         calc_delta.execute(self.load_balancer_mock, {}))
 
         vrrp_port_call = mock.call(PORT_ID)
         mock_driver.get_port.assert_has_calls([vrrp_port_call])
@@ -990,14 +985,12 @@ class TestNetworkTasks(base.TestCase):
     def test_update_vip_sg(self, mock_session, mock_lb_get,
                            mock_get_net_driver):
         mock_driver = mock.MagicMock()
-        mock_driver.update_vip_sg.return_value = SG_ID
         mock_lb_get.return_value = LB
         mock_get_net_driver.return_value = mock_driver
         net = network_tasks.UpdateVIPSecurityGroup()
 
-        sg_id = net.execute(self.load_balancer_mock)
+        net.execute(self.load_balancer_mock)
         mock_driver.update_vip_sg.assert_called_once_with(LB, LB.vip)
-        self.assertEqual(sg_id, SG_ID)
 
     def test_get_subnet_from_vip(self, mock_get_net_driver):
         mock_driver = mock.MagicMock()
@@ -1019,10 +1012,8 @@ class TestNetworkTasks(base.TestCase):
         mock_get.return_value = self.db_amphora_mock
         mock_get_net_driver.return_value = mock_driver
         net = network_tasks.PlugVIPAmphora()
-        subnet = {constants.ID: SUBNET_ID}
-        mockSubnet = mock.MagicMock()
-        mock_driver.get_subnet.return_value = mockSubnet
-        net.execute(self.load_balancer_mock, amphora, subnet)
+        mockSubnet = mock_driver.get_subnet()
+        net.execute(self.load_balancer_mock, amphora, mockSubnet)
         mock_driver.plug_aap_port.assert_called_once_with(
             LB, LB.vip, self.db_amphora_mock, mockSubnet)
 
@@ -1036,13 +1027,11 @@ class TestNetworkTasks(base.TestCase):
         mock_get.return_value = self.db_amphora_mock
         mock_get_net_driver.return_value = mock_driver
         net = network_tasks.PlugVIPAmphora()
+        mockSubnet = mock.MagicMock()
         amphora = {constants.ID: AMPHORA_ID,
                    constants.LB_NETWORK_IP: IP_ADDRESS}
-        subnet = {constants.ID: SUBNET_ID}
-        mockSubnet = mock.MagicMock()
-        mock_driver.get_subnet.return_value = mockSubnet
         net.revert(AMPS_DATA[0].to_dict(), self.load_balancer_mock,
-                   amphora, subnet)
+                   amphora, mockSubnet)
         mock_driver.unplug_aap_port.assert_called_once_with(
             LB.vip, self.db_amphora_mock, mockSubnet)
 
